<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2010-2013 Daniel Lienert <daniel@lienert.cc>, Michael Knoll <mimi@kaktusteam.de>
*  All rights reserved
*
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * Class for parsing the images core data
 *
 * @package Domain
 * @subpackage Import\MetaData
 * @author Daniel Lienert <daniel@lienert.cc>
 */
class Tx_Yag_Domain_Import_MetaData_CoreDataParser extends Tx_Yag_Domain_Import_MetaData_AbstractParser implements t3lib_Singleton {

	/**
	 * @param $filePath
	 * @return array
	 */
	public function parseCoreData($filePath) {

		$imageMagicCommand = t3lib_div::imageMagickCommand('identify', '-verbose');
		$imageMagicCommand .= ' ' . $filePath;
		t3lib_utility_Command::exec($imageMagicCommand, $result);

		$data = array();
		foreach($result as $resultLine) {
			$chunks = explode(':', $resultLine);
			$data[trim($chunks[0])] = trim($chunks[1]);
		}

		return array(
			'colorSpace'=> $this->parseColorSpace($data),
			'dpi'=> $this->parseDPI($data),
		);
	}


	/**
	 * @param $data
	 * @return mixed
	 */
	protected function parseColorSpace($data) {
		if(array_key_exists('JPEG-Colorspace-Name', $data)) return $data['JPEG-Colorspace-Name'];
		if(array_key_exists('Colorspace', $data)) return $data['Colorspace'];
	}


	/**
	 * @param $data
	 * @return int
	 */
	protected function parseDPI($data) {
		if(array_key_exists('X Resolution', $data)) {
			if(stristr($data['X Resolution'], '/')) {
				$resEquationParts = explode('/', $data['X Resolution']);
				return (int) $resEquationParts[0] / (int) $resEquationParts[1];
			} else {
				return intval($data['X Resolution']);
			}
		}

		if(array_key_exists('Resolution', $data)) return intval($data['Resolution']);
	}
}
 
?>